/* define addresses of device/registers                */
/* subtract 0x4000 from actual address to              */
/* take into account offset loaded into DS register.   */

char *piapa=0x5001,    *piapb=0x5003,    *piapc=0x5005,   *piactl=0x5007;
char *uartdr=0x4000,   *uartcr=0x4002;
char *ACheat_a     = 0x6108,   *ACheat_d     = 0x6208,
     *ACclean_a    = 0x610A,   *ACclean_d    = 0x620A,  
     *p5V_a        = 0x610E,   *p5V_d        = 0x620E,      
     *n15V_a       = 0x6110,   *n15V_d       = 0x6210,     
     *p15V_a       = 0x6112,   *p15V_d       = 0x6212,    
     *CCDtemp_lo_a = 0x6116,   *CCDtemp_lo_d = 0x6216, 
     *xtemp_a      = 0x6118,   *xtemp_d      = 0x6218,
     *ambtemp_a    = 0x611A,   *ambtemp_d    = 0x621A, 
     *dometemp_a   = 0x611C,   *dometemp_d   = 0x621C, 
     *fwtemp_a     = 0x611E,   *fwtemp_d     = 0x621E,
     *CCDtemp_hi_a = 0x6136,   *CCDtemp_hi_d = 0x6236;
char *fw_latch = 0x8001, *reset_clr = 0x9000, *reset_sel = 0xA000;


void main()
{
register unsigned char dr, cr, txbyte, lnibble, id;

   *piactl = 0x99;   /* PIA port A and C inputs, port B output */

   *uartcr = 0x0;	
   *uartcr = 0x0;    /* assume worst case UART powerup; reset */
   *uartcr = 0x0;
   *uartcr = 0x40;

   *uartcr = 0xCE;   /* 2 stop bits, parity disabled, 8 bits long, /16  */

   *uartcr = 0x33;			/* enable Tx */
   *uartdr = 0xa5;			/* tell host AFB was reset */
   while ((*uartcr & 4) != 4);

   *uartcr = 0x36;   /* enable Rx, disble Tx, DTR low, reset error,RTS low */

   id = (*piapc & 0x01) << 7;
   
   while (1)
     {
     *reset_clr = 0;

     if ((cr=*uartcr) & 0x02)   	/* if UART full, read UART */
	{
        *uartcr = 0x10;		/* disable Rx, Tx, reset errors */
        dr = *uartdr;
        txbyte = 0x20;

        if (cr & 0x38)
           txbyte |= ((cr & 0x38) >> 3);  /* if error, send what kind */
					  /* bit 0 = parity error     */
					  /* bit 1 = overrun error    */
					  /* bit 2 = framing error    */

        else if ((dr & 0x80) == id)
           {
           switch ( dr&0x7f )
	      {
	      case 0x20:		/* set filter wheel position */
	      case 0x21:
	      case 0x22:
   	      case 0x23:
	      case 0x24:
		       lnibble = dr & 0x0f;
	   	       *piapb = lnibble;
                       *fw_latch = 0;

		       while ((txbyte=(*piapa & 0x07)) != lnibble);
		       break;

	      case 0x01:		/* read AFB ID */	
		       txbyte = *piapc & 0x07;
		       break;

	      case 0x02:		/* read filter wheel position */ 
		       txbyte = *piapa & 0x07;
		       break;

	      case 0x03:		/* read internal ambient temperature */
		       *ambtemp_a = 0;
   		       *ambtemp_d = 0; 
	               txbyte = *ambtemp_d;
		       break;

	      case 0x04:		/* read external temperature */
		       *xtemp_a = 0;
		       *xtemp_d = 0;
	       	       txbyte = *xtemp_d;
		       break;

	      case 0x05:		/* read dome temperature */
		       *dometemp_a = 0;
		       *dometemp_d = 0;
	      	       txbyte = *dometemp_d;
		       break;

	      case 0x06:		/* read CCD temp. low gain (coarse) */
		       *CCDtemp_lo_a = 0;
		       *CCDtemp_lo_d = 0;
		       txbyte = *CCDtemp_lo_d;
		       break;

	      case 0x07:		/* read CCD temp. high gain (fine) */
		       *CCDtemp_hi_a = 0;
		       *CCDtemp_hi_d = 0;
		       txbyte = *CCDtemp_hi_d;
		       break;

	      case 0x08:		/* read filterwheel temperature */
		       *fwtemp_a = 0;
		       *fwtemp_d = 0;
	      	       txbyte = *fwtemp_d;
		       break;


	      case 0x09:		/* read AC across heater */
		       *ACheat_a = 0;
		       *ACheat_d = 0;
	               txbyte = *ACheat_d;
		       break;

	      case 0x0a:		/* read AC across CE unit */
		       *ACclean_a = 0;
		       *ACclean_d = 0;
		       txbyte = *ACclean_d;
		       break;

	      case 0x0b:		/* read +15V */
		       *p15V_a = 0;
		       *p15V_d = 0;
	               txbyte = *p15V_d;
		       break;

              case 0x0c:		/* read +5V */
		       *p5V_a = 0;
		       *p5V_d = 0;
	      	       txbyte = *p5V_d;
		       break;

              case 0x0d:		/* read -15V */
		       *n15V_a = 0;
		       *n15V_d = 0;
	      	       txbyte = *n15V_d;
		       break;

	      case 0x0e:		/* read fan status */
		       txbyte = *piapa & 0x80;
 		       break;

              case 0x73:		/* reset AFB */
		       *reset_sel = 0;
                       break;

              default:
		       txbyte = ' ';
		       break;
	      }

           }
           *reset_clr = 0;
	   *uartcr = 0x33;			/* enable Tx */
	   *uartdr = txbyte;
           while ((*uartcr & 4) != 4);

	*uartcr = 0x36;			/* disable Tx, enable Rx */
	}
     }
}
